@@ -330,17 +330,15 @@ typedef struct _dirtycardinfo {
 #define ENTRYFRAMESIZE(x) (ISENTRYCOMPACT(x)                            \
                            ? ((COMPACTENTRYFIELD(x, compact_frame_words_offset) & compact_frame_words_mask) << log2_ptr_bytes) \
                            : RPHEADERFRAMESIZE((uptr)(x) - size_rp_header))
-#define ENTRYOFFSET(x) (ISENTRYCOMPACT(x)                               \
-                        ? RPCOMPACTHEADERTOPLINK((uptr)(x) - size_rp_compact_header) \
-                        : RPHEADERTOPLINK((uptr)(x) - size_rp_header))
-#define ENTRYOFFSETADDR(x) (ISENTRYCOMPACT(x)                               \
-                            ? &RPCOMPACTHEADERTOPLINK((uptr)(x) - size_rp_compact_header) \
-                            : &RPHEADERTOPLINK((uptr)(x) - size_rp_header))
 #define ENTRYLIVEMASK(x) (ISENTRYCOMPACT(x)                             \
                           ? FIX(COMPACTENTRYFIELD(x, compact_frame_mask_offset)) \
                           : RPHEADERLIVEMASK((uptr)(x) - size_rp_header))
 #define ENTRYNONCOMPACTLIVEMASKADDR(x) (&RPHEADERLIVEMASK((uptr)(x) - size_rp_header))
 
+/* `top-link` must be a fixed distance from end or header, whether compact or not: */
+#define ENTRYOFFSET(x) (RPCOMPACTHEADERTOPLINK((uptr)(x) - size_rp_compact_header))
+#define ENTRYOFFSETADDR(x) (&ENTRYOFFSET(x))
+
 #define PORTFD(x) ((iptr)PORTHANDLER(x))
 #define PORTGZFILE(x) ((gzFile)(PORTHANDLER(x)))
 
@@ -518,7 +516,7 @@ typedef struct thread_gc {
 
 #define main_sweeper_index maximum_parallel_collect_threads
 
-#ifdef __MINGW32__
+#if defined(__MINGW32__) && !defined(HAND_CODED_SETJMP_SIZE)
 /* With MinGW on 64-bit Windows, setjmp/longjmp is not reliable. Using
    __builtin_setjmp/__builtin_longjmp is reliable, but
    __builtin_longjmp requires 1 as its second argument. So, allocate
@@ -544,7 +542,7 @@ typedef struct thread_gc {
 #define DOUNDERFLOW\
  &CODEIT(CLOSCODE(S_lookup_library_entry(library_dounderflow, 1)),size_rp_header)
 
-#define HEAP_VERSION_LENGTH 16
+#define HEAP_VERSION_LENGTH 24
 #define HEAP_MACHID_LENGTH 16
 #define HEAP_STAMP_LENGTH 16
 
